/*****************************************************************************
 * gyach.h
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2001 Chris Pinkham
 * Released under the terms of the GPL.
 * *NO WARRANTY*
 *
 * cpinkham@infi.net
 * http://www4.infi.net/~cpinkham/gyach/
 *****************************************************************************/


#ifndef _GYACH_H_
#define _GYACH_H_

/* This OS detection code is from yahoonix, a chat client by jalapeno_tekk */
#if defined(__FreeBSD__) || defined(__bsdi__) || defined(__386BSD__) || \
    defined(__NetBSD__) || defined(__OpenBSD__)
#	define OS_BSD
#elif defined(__linux__)
#	define OS_LINUX
#elif defined(SVR4) || defined(SYSV)
#	define OS_SYSV
#else
#	warning Unsure about host OS type. Continuing anyway.
#endif


#include <stdio.h>
#include <time.h>

/* *BSD systems supposedly need this for regex.h */
#ifdef OS_BSD
#	include <sys/types.h>
#endif

#include <regex.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#define GYACH_EMAIL		"cpinkham@infi.net"
#define GYACH_AUTHOR	"buy_more_pepsi"
#define GYACH_URL		"http://www4.infi.net/~cpinkham/gyach/"

#define YAHOO_ROOM_LIST_URL "http://chat.yahoo.com/c/roomlist.html"

extern int debug_level;
extern int debug_packets;

extern GtkWidget * login_window;
extern GtkWidget * chat_window;
extern GtkWidget * chat_status;
extern GtkWidget * chat_entry;
extern GtkWidget * chat_emotions;
extern GtkWidget * chat_users;
extern GtkWidget * chat_user_menu;
extern GtkWidget * setup_menu;
extern GtkWidget * emote_menu;
extern GtkWidget * emote_window;
extern GtkWidget * pm_window;
extern GtkWidget * pm_entry;
extern GtkWidget * pm_user;
extern GtkWidget * profile_window;
extern GtkWidget * about_box;
extern GtkWidget * setup_window;
extern GtkWidget * room_window;

extern guint st_cid;

extern time_t connect_time;

extern struct yahoo_context yahoo_ctx;

extern int  emotion_count;
extern char *user_selected;
extern char *room_selected;
extern int  emote_selected;
extern char *follow_user;

extern char *capture_filename;
extern FILE *capture_fp;

extern int  child_pid;
extern int  my_status;
extern time_t auto_away_time;
extern time_t child_timeout;

extern char *prof_text;
extern char *prof_pic;

extern int  io_callback_tag;
extern int  ping_callback_tag;
extern int  logged_in;
extern int	show_blended_colors;
extern int	show_colors;
extern int	show_fonts;
extern int	def_user;
extern int	auto_away;
extern int	auto_login;
extern int	pm_brings_back;
extern int	buffer_size;
extern int	show_html;
extern int	show_statuses;
extern int	show_enters;
extern int	profile_viewer;
extern int	ignore_on_mults;
extern int  use_bold;
extern int  use_underline;
extern int  use_italics;
extern int  font_size;
extern int  force_lowercase;
extern int  ping_minutes;
extern int  suppress_mult;
extern int  chat_window_x;
extern int  chat_window_y;
extern int  chat_window_width;
extern int  chat_window_height;
extern int  chat_text_width;
extern int  chat_user_height;
extern int  disp_auto_ignored;
extern int  remember_position;
extern int  remember_password;
extern int  limit_lfs;
extern int  auto_reply_when_away;
extern int  pm_in_sep_windows;
extern int  auto_raise_pm;
extern int  auto_close_roomlist;
extern int  chat_timestamp;
extern int  chat_timestamp_pm;
extern int  ignore_guests;
extern int  auto_reconnect;
extern int  use_proxy;
extern char *auto_reply_msg;
extern char *use_color;
extern char *custom_away_message;
extern char *browser_command;
extern char *filter_command;
extern char *font_family;
extern char *font_name;
extern char *username0;
extern char *username1;
extern char *username2;
extern char *username3;
extern char *username4;
extern char *password;
extern char *def_room;
extern char *fav_room0;
extern char *fav_room1;
extern char *fav_room2;
extern char *fav_room3;
extern char *fav_room4;
extern char *fav_room5;
extern char *fav_room6;
extern char *fav_room7;
extern char *fav_room8;
extern char *fav_room9;

extern char username[];
extern char roomname[];

extern char cfg_dir[];

extern char *proxy_host;
extern int  proxy_port;

extern int connected;
extern int valid_user;

extern GList *ignore_list;
extern GList *pm_list;
extern GList *favroom_list;
extern GList *login_list;
extern GList *friend_list;
extern GList *ofriend_list;

typedef struct pm_session {
	GtkWidget *pm_window;
	GtkWidget *pm_text;
	char *pm_user;
} PM_SESSION;

typedef struct room_entry {
	char *room_name;
	char *room_id;
	int chatters;
	int populated;
	GtkCTreeNode *temp_child;
} ROOM_ENTRY;

/* some support function prototypes */
void gyach_init();
void process_gtk_events();
void set_config();
void read_config();
void set_loaded_options();
void write_config();
void read_emotions();
void save_emotions();
int alias_find( char *str );
void strip_html_tags( char *str );
void display_profile();
void *fetch_profile( void *arg );
void *display_url( void *arg );
void edit_emote();
void lower_str( char *str );
int find_user_row( char *user );
void ignore_add( char *user );
void ignore_remove( char *user );
int ignore_check( char *user );
int auto_ignore_check( char *str );
int chars_match( char *str1, char *str2 );
void ignore_load();
void ignore_save();
void ignore_toggle( char *user );
void mute_add( char *user );
void mute_remove( char *user );
int mute_check( char *user );
void mute_toggle( char *user );
void regex_toggle( char *regex );
void dump_packet();
void convert_tags( char *str );
void strip_junk( char *str );
void tab_complete_username();
void capture_text_to_file( char *filename );
void set_setup_options();
void get_setup_options();
void populate_child_room_list( char *number, GtkCTree *room_tree,
    GtkCTreeNode *parent_node );
void populate_room_list();
void subst_escs( char *str );
void history_add( char *str );
char *history_next();
char *history_prev();
void history_print();
char *filter_text( char *txt );
GList *find_pm_session( char *user );
GList *find_pm_session_from_widget( GtkWidget *widget, char *name );
char *replace_args( char *str, char *args );
void io_callback(gpointer data, gint source, GdkInputCondition condition);
int login_to_yahoo_chat( void );
void populate_friend_list( char *friends );
void *fetch_friend_list( void *arg );
int remove_friend( char *friend );
int add_friend( char *friend );
int find_friend( char *friend );
int remove_online_friend( char *friend );
int add_online_friend( char *friend );
int find_online_friend( char *friend );
int show_friends( void );
int build_online_friends_list( void );
int set_menu_connected( int connected );


/*
 * DBG_LEVEL is defined at configuration time with the following:
 *    ./configure --enable-debug=DBG_LEVEL
 *
 * Turn ON debugging, levels are as follows:
 *   0 - none
 *   1 - main function calls
 *   2 - results of main functions
 * still working on
 *  11 - most function calls
 *  12 - results of most functions
 * and eventually
 *  21 - more function calls
 *  22 - results of all functions
 * and even later
 *  31 - all function calls
 *  32 - results of all functions
 * 201 - line numbers in lots of functions
 * 255 - ALL debug messages (including calls to process_gtk_events() function)
 */


/* debug macro */
#if DBG_LEVEL
/* DBG_LEVEL is defined so define the macro as normal */
#define DBG( level, msg, args... ) \
	do { \
		if ( DBG_LEVEL >= ( level )) { \
			fprintf( stderr, msg, ##args ); \
			fflush( stderr ); \
		} \
	} while( 0 )
#else
/* DBG_LEVEL is NOT defined so define the macro empty */
#define DBG( level, msg, args... )
#endif

#endif /* #ifndef _GYACH_H_ */
